package net.dergrimm.keycloak.providers.events.mqtt

import org.keycloak.Config
import akka.stream.alpakka.mqtt.MqttQoS

object MqttOptions:
  def fromConfig(config: Config.Scope): MqttOptions =

    val topic = config.get("topic")
    if topic == null then
      throw new IllegalArgumentException("MQTT topic is null")

    val retained = config.getBoolean("retained")

    val qos = config.getInt("qos", 0) match
      case 0 => MqttQoS.atMostOnce
      case 1 => MqttQoS.atLeastOnce
      case 2 => MqttQoS.exactlyOnce

    MqttOptions(
      topic = topic,
      retained,
      qos
    )

private final case class MqttOptions(
    topic: String,
    retained: Boolean,
    qos: MqttQoS
)
